/*$Id: s_ac_set.cc,v 26.62 2008/01/13 23:49:40 al Exp $ -*- C++ -*-
 * Copyright (C) 2001 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *------------------------------------------------------------------
 * ac analysis setup
 */
//testing=script 2006.07.14
#include "u_prblst.h"
#include "ap.h"
#include "s_ac.h"
/*--------------------------------------------------------------------------*/
//	void	AC::setup(CS&);
//	void	AC::by(CS&);
//	void	AC::decade(CS&);
//	void	AC::lin(CS&);
//	void	AC::octave(CS&);
//	void	AC::times(CS&);
/*--------------------------------------------------------------------------*/
static int needslinfix;	// flag: lin option needs patch later (spice compat)
/*--------------------------------------------------------------------------*/
void AC::setup(CS& cmd)
{
  out = IO::mstdout;
  out.reset(); //BUG// don't know why this is needed
  temp_c = OPT::temp_c;
  bool ploton = IO::plotset  &&  plotlist().size() > 0;

       if (cmd.umatch("*"))	    {untested(); times(cmd);}
  else if (cmd.umatch("by "))       {untested(); by(cmd);}
  else if (cmd.umatch("step "))     {untested(); by(cmd);}
  else if (cmd.umatch("d{ecade} ")) {		 decade(cmd);}
  else if (cmd.umatch("ti{mes} "))  {untested(); times(cmd);}
  else if (cmd.umatch("lin "))	    {untested(); lin(cmd);}
  else if (cmd.umatch("o{ctave} ")) {		 octave(cmd);}
  
  if (cmd.is_float()) {
    _start = cmd.ctof();
    _stop  = cmd.ctof();
    if (_stop==0.) {
      _stop = _start;
    }
    if (cmd.is_float()) {
      by(cmd);
    }
  }

  unsigned here = cmd.cursor();
  do{
    ONE_OF
      || (cmd.umatch("*")	  && times(cmd))
      || (cmd.umatch("+")	  && by(cmd))
      || (cmd.umatch("by ")	  && by(cmd))
      || (cmd.umatch("step ")	  && by(cmd))
      || (cmd.umatch("d{ecade} ") && decade(cmd))
      || (cmd.umatch("ti{mes} ")  && times(cmd))
      || (cmd.umatch("lin ")	  && lin(cmd))
      || (cmd.umatch("o{ctave} ") && octave(cmd))
      || Get(cmd, "dt{emp}",	   &temp_c,  mOFFSET, OPT::temp_c)
      || Get(cmd, "pl{ot}",	   &ploton)
      || Get(cmd, "pr{evoppoint}", &_prevopppoint)
      || Get(cmd, "sta{rt}",	   &_start)
      || Get(cmd, "sto{p}",	   &_stop)
      || Get(cmd, "te{mperature}", &temp_c)
      || outset(cmd,&out)
      ;
  }while (cmd.more() && !cmd.stuck(&here));
  cmd.check(bWARNING, "what's this?");

  IO::plotout = (ploton) ? IO::mstdout : OMSTREAM();
  initio(out);
  if (needslinfix) {untested();		// LIN option is # of points.
    assert(_step >= 2);			// Must compute step after 
    _step=(_stop-_start)/(_step-1.);	// reading start and stop,
    needslinfix = false;		// but step must be read first
  }					// for Spice compatibility
  if (_step==0.) {
    _step = _stop - _start;
    _linswp = true;
  }
}
/*--------------------------------------------------------------------------*/
bool AC::by(CS& cmd)
{
  _step = cmd.ctof();
  needslinfix = false;
  _linswp = true;
  return true;
}
/*--------------------------------------------------------------------------*/
bool AC::decade(CS& cmd)
{
  _step = cmd.ctopf();
  if (_step == 0.) {
    _step = 1.;
  }
  _step = pow(10., 1./_step);
  needslinfix = false;
  _linswp = false;
  return true;
}
/*--------------------------------------------------------------------------*/
bool AC::lin(CS& cmd)
{untested();
  _step = cmd.ctopf();		// need to fix step, later
  if (_step <= 2.) {untested();	// do it at the end of setup
    _step = 2.;			// a kluge, but this is a patch
  }else{untested();
  }
  needslinfix = true;		// and I am too lazy to do it
  _linswp = true;		// right.
  return true;
}
/*--------------------------------------------------------------------------*/
bool AC::octave(CS& cmd)
{
  _step = cmd.ctopf();
  if (_step == 0.) {untested();
    _step = 1.;
  }
  _step = pow(2.00000001, 1./_step);
  needslinfix = false;
  _linswp = false;
  return true;
}
/*--------------------------------------------------------------------------*/
bool AC::times(CS& cmd)
{untested();
  _step = cmd.ctopf();
  if (_step == 0.  &&  _start != 0.) {untested();
    _step = _stop / _start;
    untested();
  }else{untested();
  }
  needslinfix = false;
  _linswp = false;
  return true;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
