/*$Id: s_fo_set.cc,v 26.62 2008/01/13 23:49:40 al Exp $ -*- C++ -*-
 * Copyright (C) 2001 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *------------------------------------------------------------------
 * set up  fourier analysis
 */
//testing=script,sparse 2006.11.21
#include "u_prblst.h"
#include "ap.h"
#include "s_fo.h"
/*--------------------------------------------------------------------------*/
//	void	FOURIER::setup(CS&);
//	void	FOURIER::fftallocate();
//	void	FOURIER::fftunallocate();
static	int	to_pow_of_2(double);
/*--------------------------------------------------------------------------*/
/* fo_setup: fourier analysis: parse command string and set options
 * 	(options set by call to TRANSIENT::options)
 */
void FOURIER::setup(CS& cmd)
{
  cont = true;
  if (cmd.is_pfloat()) {
    double arg1 = cmd.ctopf();
    double arg2 = cmd.ctopf();
    double arg3 = cmd.ctopf();
    if (arg3 != 0.) {			    /* 3 args: all */
      _fstart = arg1;
      _fstop  = arg2;
      _fstep  = arg3;
    }else if (arg2 != 0.) {untested();	    /* 2 args: start = 0 */
      if (arg1 >= arg2) {untested();	    /* 2 args: stop, step */
	_fstart = 0.;		    	    /* 	(stop > step) */
	_fstop  = arg1;
	_fstep  = arg2;
      }else{untested(); /* arg1 < arg2 */    /* 2 args: step, stop */
	_fstart = 0.;
	_fstop  = arg2;
	_fstep  = arg1;
      }
    }else if (arg1 == 0.) {untested();	    /* 1 arg: start */
      _fstart = 0.;
      /* _fstop unchanged */
      /* _fstep unchanged */
    }else{untested();			    /* 1 arg: step */
      _fstart = 0.;
      _fstop  = OPT::harmonics * arg1;
      _fstep  = arg1;
    }
  }else{itested();
  }
  /* else (no args) : no change */
  
  if (_fstep == 0.) {itested();
    throw Exception("frequency step = 0");
  }else{
  }
  if (_fstop == 0.) {untested();
    _fstop = OPT::harmonics * _fstep;
  }else{
  }
  options(cmd);
  
  _timesteps = to_pow_of_2(_fstop*2 / _fstep) + 1;
  if (cold) {untested();
    cont = false;
    tstart = 0.;
  }else{
    cont = true;
    tstart = last_time;
  }
  tstop = tstart + 1. / _fstep;
  tstep = 1. / _fstep / (_timesteps-1);
  time1 = time0 = tstart;
  dtmax = tstep / static_cast<double>(skip);
  dtmin = std::max(dtmin,dtmax/dtratio);
}
/*--------------------------------------------------------------------------*/
/* allocate:  allocate space for fft
 */
void FOURIER::fftallocate()
{
  int probes = printlist().size();
  _fdata = new COMPLEX*[probes];
  for (int ii = 0;  ii < probes;  ++ii) {
    _fdata[ii] = new COMPLEX[_timesteps+100];
  }
}
/*--------------------------------------------------------------------------*/
/* unallocate:  unallocate space for fft
 */
void FOURIER::fftunallocate()
{
  if (_fdata) {
    for (int ii = 0;  ii < printlist().size();  ++ii) {
      delete [] _fdata[ii];
    }
    delete [] _fdata;
    _fdata = NULL;
  }else{itested();
  }
}
/*--------------------------------------------------------------------------*/
/* to_pow_of_2: round up to nearest power of 2
 * example: z=92 returns 128
 */
static int to_pow_of_2(double z)
{
  int x = static_cast<int>(floor(z));
  int y;
  for (y = 1; x > 0; x >>= 1) {
    y <<= 1;
  }
  return y;
}   
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
